import os_android_play_console_automation.bp._binaries_bp as _binaries_bp
from os_android_play_console_automation.bp import _images_bp
from os_android_play_console_automation.types.ImageType import ImageType
from os_android_play_console_automation.types.BinaryType import BinaryType


############################################################################################
# This module aim to automate the most common Google Play Developer actions.               #
#                                                                                          #
# To run, make sure you have the client-secrets.json file. If you don't have it,           #
# read in the GitHub repo on how to acquire it:                                            #
# [link]https://github.com/osfunapps/os-android-google-play-publisher-py#Prerequisites #
############################################################################################

def update_version(package_name, client_secrets_path, publish_percents, version_code):
    """
    Will update a released version

    Args:
        package_name: Your app's package name
        client_secrets_path: The path to your Google's client secrets json (read on how to obtain at 'os_android_apk_google_play_publisher <https://github.com/osfunapps/os-android_apk-google-play-publisher-py#Prerequisites>')
        publish_percents: Roll out percents are -> 0.1 to 10%, 0.85 to 85% etc...
        version_code: The version code of the binary (APK/App Bundle)
    """
    _binaries_bp.publish_binary(package_name, client_secrets_path, publish_percents, version_code, None, None)


def publish_apk(package_name, apk_file_path, client_secrets_path, publish_percents, version_code):
    """
    Will publish a new apk to the Google's Play Store (you HAVE to have a published app in the Google's Play Store already)

    Args:
        package_name: your app's package name
        apk_file_path: the path to your apk file.
        client_secrets_path: the path to your Google's client secrets json (read on how to obtain at 'os_android_apk_google_play_publisher <https://github.com/osfunapps/os_android_apk_google_play_publisher>')
        publish_percents: roll out publish_percents are -> 0.1 to 10%, 0.85 to 85% etc...
        version_code: the version code of the current apk
    """

    if apk_file_path is None:
        raise Exception("Error: apk_file_path is None! If you want to update an already published APK file, call 'update_version()' instead")
    else:
        _binaries_bp.publish_binary(package_name, client_secrets_path, publish_percents, version_code, apk_file_path, BinaryType.APK)


def publish_app_bundle(package_name, app_bundle_path, client_secrets_path, publish_percents, version_code):
    """
    Will publish a new App Bundle to the Google's Play Store (you HAVE to have a published app in the Google's Play Store already)

    Args:
        package_name: your app's package name
        app_bundle_path: the path to your App Bundle file.
        client_secrets_path: the path to your Google's client secrets json (read on how to obtain at 'os_android_apk_google_play_publisher <https://github.com/osfunapps/os_android_apk_google_play_publisher>')
        publish_percents: roll out publish_percents are -> 0.1 to 10%, 0.85 to 85% etc...
        version_code: the version code of the current App Bundle
    """

    if app_bundle_path is None:
        raise Exception("Error: apk_file_path is None! If you want to update an already published APK file, call 'update_version()' instead")
    else:
        _binaries_bp.publish_binary(package_name, client_secrets_path, publish_percents, version_code, app_bundle_path, BinaryType.APP_BUNDLE)


def clear_images(client_secrets_path, package_name, image_type: ImageType, language_initials='en-US', initials_as_google_translate=False):
    """
    Will clear images of a certain type from a certain app in a certain langauge

    Args:
      client_secrets_path: the path to your Google's client secrets json (read on how to obtain at 'os_android_apk_google_play_publisher <https://github.com/osfunapps/os_android_apk_google_play_publisher>')
      package_name: your app's package name
      image_type: the type of the image
      language_initials: the language in which you want to change the images
      initials_as_google_translate: if the initials are represented by the google translate api, select true. This will turn them to Google Play ones
  """
    _images_bp.clear_images(client_secrets_path, package_name, image_type, language_initials, initials_as_google_translate)


def push_images(client_secrets_path, package_name, image_type: ImageType, img_list, language_initials='en-US', initials_as_google_translate=False):
    """
    Will add a bunch of images of a certain type to a certain app in a certain langauge

    Args:
      client_secrets_path: the path to your Google's client secrets json (read on how to obtain at 'os_android_apk_google_play_publisher <https://github.com/osfunapps/os_android_apk_google_play_publisher>')
      package_name: your app's package name
      image_type: the type of the images
      language_initials: the language in which you want to change the images
      img_list: the list of paths to the images
      initials_as_google_translate: if the initials are represented by the google translate api, select true. This will turn them to Google Play ones
  """
    _images_bp.push_images(client_secrets_path, package_name, image_type, img_list, language_initials, initials_as_google_translate)


# a common executor
def _build_binary_and_run(package_name, client_secrets_path, percents, version_code, binary_file_path, binary_type):
    parsed_flags = _binaries_bp.parse_flags()

    upload = 'false'
    if binary_file_path is not None:
        upload = 'true'

    _binaries_bp.publish_binary(parsed_flags, package_name, binary_file_path, percents, version_code, client_secrets_path, upload, binary_type)



##############################################################
# just the boiler plate of the images handling actions       #
##############################################################

import argparse
from os_android_play_console_automation.bp import _shared_tools
from os_android_play_console_automation.types.ImageType import ImageType

from oauth2client import client


# will clear all of the images in the server
def clear_images(client_secrets_path, package_name, image_type: ImageType, language_initials, initials_as_google_translate=False):
    initials_to_use = language_initials
    if initials_as_google_translate:
        language_initials = _shared_tools.google_translate_to_google_play_initials(language_initials)
    try:
        service = _shared_tools.build_service(client_secrets_path)
        edit_id = _shared_tools.get_edit_id(service, package_name)

        service.edits().images().deleteall(
            packageName=package_name,
            editId=edit_id,
            language=language_initials,
            imageType=image_type.value,
        ).execute()
        print(f'All of the {image_type.value} removed for package name "{package_name}" with the language "{language_initials}"')

        _shared_tools.commit(service, edit_id, package_name)

    except client.AccessTokenRefreshError:
        print('The credentials have been revoked or expired, please re-run the '
              'application to re-authorize')


# will push an image to the server
def push_images(client_secrets_path, package_name, image_type: ImageType, img_list, language_initials='en-US', initials_as_google_translate=False):
    initials_to_use = language_initials
    if initials_as_google_translate:
        language_initials = _shared_tools.google_translate_to_google_play_initials(language_initials)
    while True:
        try:
            service = _shared_tools.build_service(client_secrets_path)
            edit_id = _shared_tools.get_edit_id(service, package_name)

            counter = 1
            for img_path in img_list:
                service.edits().images().upload(
                    packageName=package_name,
                    editId=edit_id,
                    language=language_initials,
                    imageType=image_type.value,
                    media_body=img_path
                ).execute()
                print(f'Image {str(counter)} for {image_type.value} added for package name "{package_name}" with the language "{language_initials}"')
                counter += 1

            _shared_tools.commit(service, edit_id, package_name)
            return
        except Exception as e:
            print(e)
            exception_str = str(e)
            if 'Expected OAuth 2 access token' in exception_str:
                print(f'*************************************************\n'
                      f'Got weird \'no auth session token\' error.'
                      f'\nIt seems like a bug from Google\'s side. Running again'
                      f'\nNOTICE: This error code also mean that the language {language_initials}:'
                      f'\nisn\'t in the Play Console or '
                      f'\nmaybe you should remove the .dat file and refresh the login params'
                      f'\n*************************************************')
            else:
                raise e



import enum


class ImageType(enum.Enum):
    PHONE_SCREENSHOTS = 'phoneScreenshots'
    SEVEN_INCH_SCREENSHOTS = 'sevenInchScreenshots'
    TEN_INCH_SCREENSHOTS = 'tenInchScreenshots'
    TV_SCREENSHOTS = 'tvScreenshots'
    WEAR_SCREENSHOTS = 'wearScreenshots'
    ICON = 'icon'
    FEATURE_GRAPHIC = 'featureGraphic'
    TV_BANNER = 'tvBanner'



from os_android_play_console_automation.bp import sample_tools

PLAY_INITIALS_BANK = ['en-GB', 'af', 'am', 'ar', 'hy-AM', 'az-AZ', 'bn-BD', 'eu-ES', 'be', 'bg', 'my-MM', 'ca', 'zh-HK', 'zh-CN', 'zh-TW', 'hr', 'cs-CZ', 'da-DK', 'nl-NL', 'en-AU', 'en-IN', 'en-SG',
                      'en-ZA', 'en-CA', 'en-US', 'et', 'fil', 'fi-FI', 'fr-FR', 'fr-CA', 'gl-ES', 'ka-GE', 'de-DE', 'el-GR', 'hi-IN', 'hu-HU', 'is-IS', 'id', 'it-IT', 'ja-JP', 'kn-IN', 'km-KH',
                      'ko-KR', 'ky-KG', 'lo-LA', 'lv', 'lt', 'mk-MK', 'ms', 'ml-IN', 'mr-IN', 'mn-MN', 'ne-NP', 'no-NO', 'fa', 'pl-PL', 'pt-BR',
                      'pt-PT', 'ro', 'ro', 'ru-RU', 'sr', 'si-LK', 'sk', 'sl', 'es-419', 'es-ES', 'es-US', 'sw', 'sv-SE', 'ta-IN', 'te-IN', 'th', 'tr-TR', 'uk', 'vi', 'zu']


# will build the service
def build_service(client_secrets_path):
    args = [str(__file__)]

    # Authenticate and construct service.
    service, flags = sample_tools.init(
        args,
        'androidpublisher',
        'v3',
        __doc__,
        client_secrets_path,
        parents=[],
        scope='https://www.googleapis.com/auth/androidpublisher')

    # fix google's shitty algorithm of calculating timeout of the http driver to make sure the apk uploaded successfully
    service._http.timeout = 300
    return service


# will get the edit id from the execute
def get_edit_id(service, package_name):
    edit_request = service.edits().insert(body={}, packageName=package_name)
    result = edit_request.execute()
    return result['id']


# will commit the request to Edit's api
def commit(service, edit_id, package_name):
    commit_request = service.edits().commit(editId=edit_id, packageName=package_name).execute()
    print('Edit "%s" has been committed' % (commit_request['id']))


# will turn a google translate language initials to Google Play translation initials
def google_translate_to_google_play_initials(translate_initials):
    if translate_initials == 'pt':
        print(f'Notice: Got the initials "pt". By default returning pt-brazil')
        return 'pt-BR'
    elif translate_initials == 'es':
        return 'es-ES'
    elif translate_initials == 'ur':
        return 'ur'
    elif translate_initials == 'kk':
        return 'kk'
    elif translate_initials == 'tl':
        return 'fil'
    elif translate_initials == 'he':
        return 'iw-IL'
    elif translate_initials == 'in':
        return 'id'
    elif translate_initials == 'ur':
        return 'uk'
    elif translate_initials == 'en':
        print(f'Notice: Got the initials "en". By default returning en-US')
        return 'en-US'
    elif translate_initials == 'zh':
        print(f'Notice: Got the initials "zh". By default returning zh-CN')
        return 'zh-CN'
    play_initials = next((play_lang for play_lang in PLAY_INITIALS_BANK if play_lang.startswith(translate_initials)), None)
    return play_initials



##############################################################
# just the boiler plate of the apk/app bundle publisher file #
##############################################################

import argparse
from os_android_play_console_automation.bp import sample_tools
from oauth2client import client
from os_android_play_console_automation.bp import _shared_tools
from os_android_play_console_automation.types.BinaryType import BinaryType


#
#
# def publish_binary(parsed_flags, package_name, binary_file_path, percents, version_code, client_secrets_path, upload, binary_type):
#     args = [str(__file__), package_name, binary_file_path, str(percents), 'production', str(version_code), upload]
#
#     # Authenticate and construct service.
#     service, flags = sample_tools.init(
#         args,
#         'androidpublisher',
#         'v3',
#         __doc__,
#         client_secrets_path,
#         parents=[parsed_flags],
#         scope='https://www.googleapis.com/auth/androidpublisher')
#
#     # fix google's shitty algorithm of calculating timeout of the http driver to make sure the apk uploaded successfully
#     for key, val in service._http.connections.items():
#         val.sock.settimeout(260)
#
#     # Process flags and read their values.
#     package_name = flags.package_name
#     binary_file = flags.apk_file
#     user_fraction = float(flags.user_fraction)
#
#     track = flags.track
#     version_code = float(flags.version_code)
#     upload = flags.upload
#
#     try:
#
#         # service = _shared_tools.build_service(client_secrets_path)
#         # edit_id = _shared_tools.get_edit_id(service, package_name)
#
#         edit_request = service.edits().insert(body={}, packageName=package_name)
#         result = edit_request.execute()
#         edit_id = result['id']
#
#         if upload == 'true':
#             print("uploading...")
#
#             if binary_type == BinaryType.APK:
#                 binary_response = service.edits().apks().upload(
#                     editId=edit_id,
#                     packageName=package_name,
#                     media_body=binary_file_path
#                 ).execute()
#             elif binary_type == BinaryType.APP_BUNDLE:
#                 binary_response = service.edits().bundles().upload(
#                     editId=edit_id,
#                     packageName=package_name,
#                     media_body=binary_file_path,
#                     media_mime_type="application/octet-stream"
#                 ).execute()
#             else:
#                 raise Exception(f'Binary type {binary_type} not supported!')
#
#             version_code = binary_response['versionCode']
#             print('Version code %d has been uploaded' % binary_response['versionCode'])
#
#         track_response = service.edits().tracks().update(
#             editId=edit_id,
#             track=percents,
#             packageName=package_name,
#             body={u'releases': [obtain_json(user_fraction, version_code)]}).execute()
#
#         print('Track %s is set with releases: %s' % (
#             track_response['track'], str(track_response['releases'])))
#
#         commit_request = service.edits().commit(
#             editId=edit_id, packageName=package_name).execute()
#
#         print('Edit "%s" has been committed' % (commit_request['id']))
#
#     except client.AccessTokenRefreshError:
#         print('The credentials have been revoked or expired, please re-run the '
#               'application to re-authorize')
#
#
# def obtain_json(user_fraction, version_code):
#     if user_fraction != 1:
#         return {
#             u'name': u'Automated release',
#             u'versionCodes': [version_code],
#             u'userFraction': user_fraction,
#             u'status': u'inProgress'
#         }
#
#     else:
#         return {
#             u'name': u'Automated release',
#             u'versionCodes': [version_code],
#             u'status': u'completed'
#         }
#

def publish_binary(package_name, client_secrets_path, publish_percents, version_code, binary_file_path, binary_type: BinaryType):
    try:

        service = _shared_tools.build_service(client_secrets_path)
        edit_id = _shared_tools.get_edit_id(service, package_name)

        if binary_file_path is not None:
            print("uploading...")

            if binary_type == BinaryType.APK:
                binary_response = service.edits().apks().upload(
                    editId=edit_id,
                    packageName=package_name,
                    media_body=binary_file_path
                ).execute()
            elif binary_type == BinaryType.APP_BUNDLE:
                binary_response = service.edits().bundles().upload(
                    editId=edit_id,
                    packageName=package_name,
                    media_body=binary_file_path,
                    media_mime_type="application/octet-stream"
                ).execute()
            else:
                raise Exception(f'Binary type {binary_type} not supported!')

            version_code = binary_response['versionCode']
            print('Version code %d has been uploaded' % binary_response['versionCode'])

        track_response = service.edits().tracks().update(
            editId=edit_id,
            track='production',
            packageName=package_name,
            body={u'releases': [obtain_json(publish_percents, version_code)]}).execute()

        print('Track %s is set with releases: %s' % (
            track_response['track'], str(track_response['releases'])))

        _shared_tools.commit(service, edit_id, package_name)

    except client.AccessTokenRefreshError:
        print('The credentials have been revoked or expired, please re-run the '
              'application to re-authorize')


def obtain_json(user_fraction, version_code):
    if user_fraction != 1:
        return {
            u'name': u'Automated release',
            u'versionCodes': [version_code],
            u'userFraction': user_fraction,
            u'status': u'inProgress'
        }

    else:
        return {
            u'name': u'Automated release',
            u'versionCodes': [version_code],
            u'status': u'completed'
        }


# # # will parse the required flags for the roll out
# def parse_flags():
#     arg_parser = argparse.ArgumentParser(add_help=False)
#     arg_parser.add_argument('package_name',
#                             help='The package name. Example: com.android.sample')
#     arg_parser.add_argument('apk_file',
#                             nargs='?',
#                             default='apk.apk',
#                             help='The path to the APK file to upload.')
#     arg_parser.add_argument('user_fraction',
#                             nargs='?',
#                             default='0.0',
#                             help='user percentage')
#     arg_parser.add_argument('track',
#                             nargs='?',
#                             default='0.0',
#                             help='should be wither roll out or production')
#     arg_parser.add_argument('version_code',
#                             nargs='?',
#                             default='0.0',
#                             help='version code')
#     arg_parser.add_argument('upload',
#                             nargs='?',
#                             default='0.0',
#                             help='should upload the apk or not')
#     return arg_parser

# # LOG:
# # sample_tools.py is a local copy of Google's sample_tools.py file. This file has been added a cache_discovery=False in:
# # service = discovery.build(name, version, http=http, cache_discovery=False) in order to remember the last dat file



from distutils.core import setup

setup(
    name='os_android_play_console_automation',
    packages=['os_android_play_console_automation',
              'os_android_play_console_automation.types',
              'os_android_play_console_automation.bp'],
    version='1.10',
    license='MIT',
    description='This module will automate a bunch of Google Play actions like publish apk/app bundle and image handling',  # Give a short description about your library
    author='Oz Shabat',
    author_email='admin@os-apps.com',
    url='https://github.com/osfunapps/os-android-play-console-automation-py',  # Provide either the link to your github or to your website
    keywords=['python', 'osfunapps', 'apk', 'google_play', 'play_console', 'app_bundle', 'android', 'automation', 'release', 'publish', 'assemble-release', 'create', 'google-play', 'screenshots', 'logo', 'feature_graphic'],  # Keywords that define your package best
    install_requires=['google-api-python-client', 'oauth2client'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.9',
    ],
)
